@@ -115,6 +115,108 @@ TEST(block_folded, empty_block1)
     EXPECT_EQ(t[4].val(), csubstr(""));
 }
 
+TEST(block_folded, empty_block_as_container_member)
+{
+    // this was ok
+    test_check_emit_check(R"(
+map:
+  a: ""
+  b: ''
+  d: |
+  c: >
+  e:
+)", [](Tree const &t){
+        EXPECT_TRUE(t["map"].has_key());
+        EXPECT_TRUE(t["map"].is_map());
+        EXPECT_EQ(t["map"].num_children(), 5u);
+        for(const auto &child : t["map"].children())
+        {
+            EXPECT_EQ(child.val(), "");
+            if(child.key() != "e")
+            {
+                EXPECT_TRUE(child.type().is_val_quoted());
+                EXPECT_FALSE(child.val_is_null());
+            }
+        }
+    });
+    // this was ok
+    test_check_emit_check(R"(
+map:
+  a: ""
+  b: ''
+  d: |
+  c: >
+)", [](Tree const &t){
+        EXPECT_TRUE(t["map"].has_key());
+        EXPECT_TRUE(t["map"].is_map());
+        EXPECT_TRUE(t["map"].is_map());
+        EXPECT_EQ(t["map"].num_children(), 4u);
+        for(const auto &child : t["map"].children())
+        {
+            EXPECT_EQ(child.val(), "");
+            EXPECT_TRUE(child.type().is_val_quoted());
+            EXPECT_FALSE(child.val_is_null());
+        }
+    });
+    // this was not ok! the block literal before next is extended: to
+    // include the YAML for next!
+    test_check_emit_check(R"(
+map:
+  a: ""
+  b: ''
+  d: |
+  c: >
+next:
+  a: ""
+  b: ''
+  d: |
+  c: >
+)", [](Tree const &t){
+        for(const char *name : {"map", "next"})
+        {
+            ASSERT_TRUE(t.rootref().has_child(to_csubstr(name))) << "name=" << name;
+            ConstNodeRef node = t[to_csubstr(name)];
+            EXPECT_TRUE(node.has_key());
+            EXPECT_TRUE(node.is_map());
+            EXPECT_TRUE(node.is_map());
+            ASSERT_EQ(node.num_children(), 4u);
+            for(const auto &child : node.children())
+            {
+                EXPECT_EQ(child.val(), "");
+                EXPECT_TRUE(child.type().is_val_quoted());
+                EXPECT_FALSE(child.val_is_null());
+            }
+        }
+    });
+    test_check_emit_check(R"(
+seq:
+  - ""
+  - ''
+  - |
+  - >
+next:
+  - ""
+  - ''
+  - |
+  - >
+)", [](Tree const &t){
+        for(const char *name : {"seq", "next"})
+        {
+            ASSERT_TRUE(t.rootref().has_child(to_csubstr(name))) << "name=" << name;
+            ConstNodeRef node = t[to_csubstr(name)];
+            EXPECT_TRUE(node.has_key());
+            EXPECT_TRUE(node.is_seq());
+            ASSERT_EQ(node.num_children(), 4u);
+            for(const auto &child : node.children())
+            {
+                EXPECT_EQ(child.val(), "");
+                EXPECT_TRUE(child.type().is_val_quoted());
+                EXPECT_FALSE(child.val_is_null());
+            }
+        }
+    });
+}
+
 TEST(block_folded, issue152_not_indented)
 {
     const Tree t = parse_in_arena(R"(
@@ -164,7 +266,7 @@ indented_once:
       exec pg_isready -U "dog" -d "dbname=dog" -h 127.0.0.1 -p 5432
   err3_parses: no
 )");
-    const NodeRef n = t["indented_once"];
+    ConstNodeRef n = t["indented_once"];
     EXPECT_EQ(n["ok"  ][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
     EXPECT_EQ(n["err" ][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
     EXPECT_EQ(n["err2"][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
@@ -194,7 +296,7 @@ indented_once:
         exec pg_isready -U "dog" -d "dbname=dog" -h 127.0.0.1 -p 5432
     err3_parses: no
 )");
-    const NodeRef n = t["indented_once"]["indented_twice"];
+    ConstNodeRef n = t["indented_once"]["indented_twice"];
     EXPECT_EQ(n["ok"  ][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
     EXPECT_EQ(n["err" ][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
     EXPECT_EQ(n["err2"][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
@@ -225,7 +327,7 @@ indented_once:
           exec pg_isready -U "dog" -d "dbname=dog" -h 127.0.0.1 -p 5432
       err3_parses: no
 )");
-    const NodeRef n = t["indented_once"]["indented_twice"]["indented_thrice"];
+    ConstNodeRef n = t["indented_once"]["indented_twice"]["indented_thrice"];
     EXPECT_EQ(n["ok"  ][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
     EXPECT_EQ(n["err" ][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
     EXPECT_EQ(n["err2"][0].val(), csubstr("exec pg_isready -U \"dog\" -d \"dbname=dog\" -h 127.0.0.1 -p 5432\n"));
@@ -283,7 +385,7 @@ foo:
 )";
     test_check_emit_check(yaml, [](Tree const &t){
         ASSERT_TRUE(t.rootref().is_stream());
-        NodeRef doc = t.rootref().child(0);
+        ConstNodeRef doc = t.rootref().child(0);
         ASSERT_TRUE(doc.is_seq());
         ASSERT_EQ(doc.num_children(), 4u);
         EXPECT_EQ(doc[0].val(), csubstr("child0\n"));
@@ -644,7 +746,7 @@ TEST(block_folded, test_suite_W4TN)
 # empty
 )";
     test_check_emit_check(yaml, [](Tree const &t){
-        NodeRef const r = t.rootref();
+        ConstNodeRef r = t.rootref();
         ASSERT_TRUE(r.is_stream());
         ASSERT_EQ(r.num_children(), 10u);
         ASSERT_TRUE(r.doc(0).is_doc());
@@ -1130,6 +1232,13 @@ another: text
       })
 );
 
+ADD_CASE_TO_GROUP("block folded with tab and spaces",
+R"(>
+       	  )",
+  N(DOCVAL|VALQUO, "\t  \n")
+    );
+
+
 ADD_CASE_TO_GROUP("block folded with empty docval 1",
 R"(>)",
   N(DOCVAL|VALQUO, "")
@@ -1157,19 +1266,6 @@ R"(>
 ADD_CASE_TO_GROUP("block folded with empty docval 5",
 R"(>
     
-)",
-  N(DOCVAL|VALQUO, "")
-    );
-
-ADD_CASE_TO_GROUP("block folded with empty docval 6",
-R"(>
-       	  )",
-  N(DOCVAL|VALQUO, "")
-    );
-
-ADD_CASE_TO_GROUP("block folded with empty docval 7",
-R"(>
-       	  
 )",
   N(DOCVAL|VALQUO, "")
     );
@@ -1177,7 +1273,6 @@ R"(>
 ADD_CASE_TO_GROUP("block folded with empty docval 8",
 R"(>
 
-
 )",
   N(DOCVAL|VALQUO, "")
     );
