@@ -62,6 +62,108 @@ TEST(block_literal, empty_block)
 }
 
 
+TEST(block_literal, empty_block_as_container_member)
+{
+    // this was ok
+    test_check_emit_check(R"(
+map:
+  a: ""
+  b: ''
+  c: >
+  d: |
+  e:
+)", [](Tree const &t){
+        EXPECT_TRUE(t["map"].has_key());
+        EXPECT_TRUE(t["map"].is_map());
+        EXPECT_EQ(t["map"].num_children(), 5u);
+        for(const auto &child : t["map"].children())
+        {
+            EXPECT_EQ(child.val(), "");
+            if(child.key() != "e")
+            {
+                EXPECT_TRUE(child.type().is_val_quoted());
+                EXPECT_FALSE(child.val_is_null());
+            }
+        }
+    });
+    // this was ok
+    test_check_emit_check(R"(
+map:
+  a: ""
+  b: ''
+  c: >
+  d: |
+)", [](Tree const &t){
+        EXPECT_TRUE(t["map"].has_key());
+        EXPECT_TRUE(t["map"].is_map());
+        EXPECT_TRUE(t["map"].is_map());
+        EXPECT_EQ(t["map"].num_children(), 4u);
+        for(const auto &child : t["map"].children())
+        {
+            EXPECT_EQ(child.val(), "");
+            EXPECT_TRUE(child.type().is_val_quoted());
+            EXPECT_FALSE(child.val_is_null());
+        }
+    });
+    // this was not ok! the block literal before next is extended: to
+    // include the YAML for next!
+    test_check_emit_check(R"(
+map:
+  a: ""
+  b: ''
+  c: >
+  d: |
+next:
+  a: ""
+  b: ''
+  c: >
+  d: |
+)", [](Tree const &t){
+        for(const char *name : {"map", "next"})
+        {
+            ASSERT_TRUE(t.rootref().has_child(to_csubstr(name))) << "name=" << name;
+            ConstNodeRef node = t[to_csubstr(name)];
+            EXPECT_TRUE(node.has_key());
+            EXPECT_TRUE(node.is_map());
+            ASSERT_EQ(node.num_children(), 4u);
+            for(const auto &child : node.children())
+            {
+                EXPECT_EQ(child.val(), "");
+                EXPECT_TRUE(child.type().is_val_quoted());
+                EXPECT_FALSE(child.val_is_null());
+            }
+        }
+    });
+    test_check_emit_check(R"(
+seq:
+  - ""
+  - ''
+  - >
+  - |
+next:
+  - ""
+  - ''
+  - >
+  - |
+)", [](Tree const &t){
+        for(const char *name : {"seq", "next"})
+        {
+            ASSERT_TRUE(t.rootref().has_child(to_csubstr(name))) << "name=" << name;
+            ConstNodeRef node = t[to_csubstr(name)];
+            EXPECT_TRUE(node.has_key());
+            EXPECT_TRUE(node.is_seq());
+            ASSERT_EQ(node.num_children(), 4u);
+            for(const auto &child : node.children())
+            {
+                EXPECT_EQ(child.val(), "");
+                EXPECT_TRUE(child.type().is_val_quoted());
+                EXPECT_FALSE(child.val_is_null());
+            }
+        }
+    });
+}
+
+
 TEST(block_literal, emit_does_not_add_lines_to_multi_at_end_1)
 {
     Tree t = parse_in_arena("[]");
@@ -69,19 +171,19 @@ TEST(block_literal, emit_does_not_add_lines_to_multi_at_end_1)
     r.append_child() = "\n\n";
     r.append_child() = "\n\n";
     r.append_child() = "last";
-    std::string out = emitrs<std::string>(t);
+    std::string out = emitrs_yaml<std::string>(t);
     t.clear();
     t = parse_in_arena(to_csubstr(out));
     EXPECT_EQ(t[0].val(), csubstr("\n\n"));
     EXPECT_EQ(t[1].val(), csubstr("\n\n"));
     EXPECT_EQ(t[2].val(), csubstr("last"));
-    out = emitrs<std::string>(t);
+    out = emitrs_yaml<std::string>(t);
     t.clear();
     t = parse_in_arena(to_csubstr(out));
     EXPECT_EQ(t[0].val(), csubstr("\n\n"));
     EXPECT_EQ(t[1].val(), csubstr("\n\n"));
     EXPECT_EQ(t[2].val(), csubstr("last"));
-    out = emitrs<std::string>(t);
+    out = emitrs_yaml<std::string>(t);
     t.clear();
     t = parse_in_arena(to_csubstr(out));
     EXPECT_EQ(t[0].val(), csubstr("\n\n"));
@@ -104,15 +206,15 @@ TEST(block_literal, emit_does_not_add_lines_to_multi_at_end_2)
    
 
 )";
-    std::string out = emitrs<std::string>(t);
+    std::string out = emitrs_yaml<std::string>(t);
     EXPECT_EQ(out, expected);
     t = parse_in_arena(to_csubstr(out));
     EXPECT_EQ(t.docref(0).val(), csubstr("ab\n\n \n"));
-    out = emitrs<std::string>(t);
+    out = emitrs_yaml<std::string>(t);
     EXPECT_EQ(out, expected);
     t = parse_in_arena(to_csubstr(out));
     EXPECT_EQ(t.docref(0).val(), csubstr("ab\n\n \n"));
-    out = emitrs<std::string>(t);
+    out = emitrs_yaml<std::string>(t);
     EXPECT_EQ(out, expected);
     t = parse_in_arena(to_csubstr(out));
     EXPECT_EQ(t.docref(0).val(), csubstr("ab\n\n \n"));
@@ -158,12 +260,12 @@ TEST(block_literal, emit_does_not_add_lines_to_multi_at_end_3)
     Tree t = parse_in_arena(to_csubstr(yaml));
     EXPECT_EQ(t[0].val(), "Several lines of text,\nwith some \"quotes\" of various 'types',\nand also a blank line:\n\nplus another line at the end.\n");
     EXPECT_EQ(t[1].val(), "Several lines of text,\nwith some \"quotes\" of various 'types',\nand also a blank line:\n\nplus another line at the end.\n\n");
-    std::string out = emitrs<std::string>(t);
+    std::string out = emitrs_yaml<std::string>(t);
     EXPECT_EQ(out, expected);
     t = parse_in_arena(to_csubstr(out));
     EXPECT_EQ(t[0].val(), "Several lines of text,\nwith some \"quotes\" of various 'types',\nand also a blank line:\n\nplus another line at the end.\n");
     EXPECT_EQ(t[1].val(), "Several lines of text,\nwith some \"quotes\" of various 'types',\nand also a blank line:\n\nplus another line at the end.\n\n");
-    out = emitrs<std::string>(t);
+    out = emitrs_yaml<std::string>(t);
     EXPECT_EQ(out, expected);
 }
 
@@ -178,7 +280,7 @@ TEST(block_literal, carriage_return)
     Tree t = parse_in_arena(to_csubstr(yaml));
     EXPECT_EQ(t["with"].val(), "text\n \tlines\n");
     EXPECT_EQ(t["without"].val(), "text\n \tlines\n");
-    auto emitted = emitrs<std::string>(t);
+    auto emitted = emitrs_yaml<std::string>(t);
     #ifdef RYML_DBG
     __c4presc(emitted.data(), emitted.size());
     #endif
@@ -734,6 +836,13 @@ R"(
   L{N(L{N(QV, "aaa", "xxx\n"), N(QV, "bbb", "yyy\n")})}
     );
 
+ADD_CASE_TO_GROUP("block literal with tab and spaces",
+R"(|
+       	  )",
+  N(DOCVAL|VALQUO, "\t  \n")
+    );
+
+
 ADD_CASE_TO_GROUP("block literal with empty docval 1",
 R"(|)",
   N(DOCVAL|VALQUO, "")
@@ -761,19 +870,6 @@ R"(|
 ADD_CASE_TO_GROUP("block literal with empty docval 5",
 R"(|
     
-)",
-  N(DOCVAL|VALQUO, "")
-    );
-
-ADD_CASE_TO_GROUP("block literal with empty docval 6",
-R"(|
-       	  )",
-  N(DOCVAL|VALQUO, "")
-    );
-
-ADD_CASE_TO_GROUP("block literal with empty docval 7",
-R"(|
-       	  
 )",
   N(DOCVAL|VALQUO, "")
     );
