@@ -9,14 +9,19 @@
 #endif
 
 #include <gtest/gtest.h>
+#ifndef C4_EXCEPTIONS
+#include <csetjmp>
+#endif
 
 #if defined(_MSC_VER)
 #   pragma warning(push)
 #elif defined(__clang__)
 #   pragma clang diagnostic push
+#   pragma clang diagnostic ignored "-Wold-style-cast"
 #elif defined(__GNUC__)
 #   pragma GCC diagnostic push
 #   pragma GCC diagnostic ignored "-Wuseless-cast"
+#   pragma GCC diagnostic ignored "-Wold-style-cast"
 #   if __GNUC__ >= 6
 #       pragma GCC diagnostic ignored "-Wnull-dereference"
 #   endif
@@ -163,18 +168,24 @@ std::string format_error(const char* msg, size_t len, Location loc)
     #endif
     if(!loc) return msg;
     std::string out;
-    if(!loc.name.empty()) c4::formatrs(append, &out, "{}:", loc.name);
-    c4::formatrs(append, &out, "{}:{}:", loc.line, loc.col);
-    if(loc.offset) c4::formatrs(append, &out, " (@{}B):", loc.offset);
-    c4::formatrs(append, &out, "{}:", csubstr(msg, len));
+    if(!loc.name.empty()) c4::formatrs_append(&out, "{}:", loc.name);
+    c4::formatrs_append(&out, "{}:{}:", loc.line, loc.col);
+    if(loc.offset) c4::formatrs_append(&out, " (@{}B):", loc.offset);
+    c4::formatrs_append(&out, "{}:", csubstr(msg, len));
     return out;
 }
 
-struct ExpectedError : public std::runtime_error
+struct ExpectedError C4_IF_EXCEPTIONS_( : public std::runtime_error, )
 {
+    C4_IF_EXCEPTIONS_( ,
+        std::string smsg;
+        const char *what() const { return smsg.c_str(); }
+        ExpectedError() = default;
+    )
     Location error_location;
     ExpectedError(const char* msg, size_t len, Location loc)
-        : std::runtime_error(format_error(msg, len, loc))
+        : C4_IF_EXCEPTIONS_(std::runtime_error(format_error(msg, len, loc)),
+                            smsg(format_error(msg, len, loc)))
         , error_location(loc)
     {
     }
@@ -182,44 +193,72 @@ struct ExpectedError : public std::runtime_error
 
 
 //-----------------------------------------------------------------------------
+C4_IF_EXCEPTIONS_(
+    ,
+    std::jmp_buf s_jmp_env_expect_error = {};
+    ExpectedError s_jmp_err = {};
+    )
 
 ExpectError::ExpectError(Tree *tree, Location loc)
     : m_got_an_error(false)
     , m_tree(tree)
     , m_glob_prev(get_callbacks())
-    , m_tree_prev(tree->callbacks())
+    , m_tree_prev(tree ? tree->callbacks() : Callbacks{})
     , expected_location(loc)
 {
-    auto err = [](const char* msg, size_t len, Location errloc, void *this_) {
+    auto err = [](const char* msg, size_t len, Location errloc, void *this_)  {
         ((ExpectError*)this_)->m_got_an_error = true;
-        throw ExpectedError(msg, len, errloc);
+        C4_IF_EXCEPTIONS(
+            throw ExpectedError(msg, len, errloc);
+            C4_UNREACHABLE_AFTER_ERR();
+            ,
+            s_jmp_err = ExpectedError(msg, len, errloc);
+            std::longjmp(s_jmp_env_expect_error, 1);
+        );
     };
     #ifdef RYML_NO_DEFAULT_CALLBACKS
     c4::yml::Callbacks tcb((void*)this, nullptr, nullptr, err);
     c4::yml::Callbacks gcb((void*)this, nullptr, nullptr, err);
     #else
-    c4::yml::Callbacks tcb((void*)this, m_tree_prev.m_allocate, m_tree_prev.m_free, err);
+    c4::yml::Callbacks tcb((void*)this, tree ? m_tree_prev.m_allocate : nullptr, tree ? m_tree_prev.m_free : nullptr, err);
     c4::yml::Callbacks gcb((void*)this, m_glob_prev.m_allocate, m_glob_prev.m_free, err);
     #endif
-    tree->callbacks(tcb);
+    if(tree)
+        tree->callbacks(tcb);
     set_callbacks(gcb);
 }
 
 ExpectError::~ExpectError()
 {
-    m_tree->callbacks(m_tree_prev);
+    if(m_tree)
+        m_tree->callbacks(m_tree_prev);
     set_callbacks(m_tree_prev);
 }
 
+void ExpectError::check_success(Tree *tree, std::function<void()> fn)
+{
+    auto context = ExpectError(tree, {});
+    C4_IF_EXCEPTIONS_(try, if(setjmp(s_jmp_env_expect_error) == 0))
+    {
+        fn();
+    }
+    C4_IF_EXCEPTIONS_(catch(ExpectedError const&), else)
+    {
+        ;
+    }
+    EXPECT_FALSE(context.m_got_an_error);
+}
+
 void ExpectError::do_check(Tree *tree, std::function<void()> fn, Location expected_location)
 {
     auto context = ExpectError(tree, expected_location);
-    try
+    C4_IF_EXCEPTIONS_(try, if(setjmp(s_jmp_env_expect_error) == 0))
     {
         fn();
     }
-    catch(ExpectedError const& e)
+    C4_IF_EXCEPTIONS_(catch(ExpectedError const& e), else)
     {
+        C4_IF_EXCEPTIONS_( , ExpectedError const& e = s_jmp_err);
         #if defined(RYML_DBG)
         std::cout << "---------------\n";
         std::cout << "got an expected error:\n" << e.what() << "\n";
@@ -332,7 +371,7 @@ NodeType_e CaseNode::_guess() const
 
 
 //-----------------------------------------------------------------------------
-void CaseNode::compare_child(yml::NodeRef const& n, size_t pos) const
+void CaseNode::compare_child(yml::ConstNodeRef const& n, size_t pos) const
 {
     EXPECT_TRUE(pos < n.num_children());
     EXPECT_TRUE(pos < children.size());
@@ -344,8 +383,8 @@ void CaseNode::compare_child(yml::NodeRef const& n, size_t pos) const
 
     if(type & MAP)
     {
-        auto actualch = n.find_child(expectedch.key);
-        if(actualch != nullptr)
+        ConstNodeRef actualch = n.find_child(expectedch.key);
+        if(!actualch.invalid())
         {
             // there may be duplicate keys.
             if(actualch.id() != n[pos].id())
@@ -412,7 +451,7 @@ void CaseNode::compare_child(yml::NodeRef const& n, size_t pos) const
     }
 }
 
-void CaseNode::compare(yml::NodeRef const& actual, bool ignore_quote) const
+void CaseNode::compare(yml::ConstNodeRef const& actual, bool ignore_quote) const
 {
     if(ignore_quote)
     {
@@ -457,7 +496,7 @@ void CaseNode::compare(yml::NodeRef const& actual, bool ignore_quote) const
             compare_child(actual, ic++);
         }
 
-        if(actual.first_child() != nullptr)
+        if(!actual.first_child().invalid())
         {
             ic = 0;
             for(auto const ch : actual.first_child().siblings())
@@ -499,12 +538,12 @@ void CaseNode::recreate(yml::NodeRef *n) const
 
 //-----------------------------------------------------------------------------
 
-void print_path(NodeRef const& n)
+void print_path(ConstNodeRef const& n)
 {
     size_t len = 0;
     char buf[1024];
-    NodeRef p = n;
-    while(p != nullptr)
+    ConstNodeRef p = n;
+    while(p.readable())
     {
         if(p.has_key())
         {
@@ -521,7 +560,7 @@ void print_path(NodeRef const& n)
     C4_ASSERT(len < sizeof(buf));
     size_t pos = len;
     p = n;
-    while(p.valid() && p != nullptr)
+    while(p.readable())
     {
         if(p.has_key())
         {
@@ -550,7 +589,7 @@ void print_path(NodeRef const& n)
 
 void print_node(CaseNode const& p, int level)
 {
-    printf("%*s%p", (2*level), "", (void*)&p);
+    printf("%*s%p", (2*level), "", (void const*)&p);
     if( ! p.parent)
     {
         printf(" [ROOT]");
@@ -611,10 +650,10 @@ void print_node(CaseNode const& p, int level)
 }
 
 
-void print_tree(NodeRef const& p, int level)
+void print_tree(ConstNodeRef const& p, int level)
 {
     print_node(p, level);
-    for(NodeRef const ch : p.children())
+    for(ConstNodeRef ch : p.children())
     {
         print_tree(ch, level+1);
     }
@@ -635,104 +674,109 @@ void print_tree(CaseNode const& t)
     printf("--------------------------------------\n");
 }
 
-void test_invariants(NodeRef const n)
+void test_invariants(ConstNodeRef const& n)
 {
+    #define _MORE_INFO << "id=" << n.id()
+
     if(n.is_root())
     {
-        EXPECT_FALSE(n.has_other_siblings());
+        EXPECT_FALSE(n.has_other_siblings()) _MORE_INFO;
     }
     // keys or vals cannot be root
     if(n.has_key() || n.is_val() || n.is_keyval())
     {
-        EXPECT_TRUE(!n.is_root() || (n.is_doc() && !n.has_key()));
+        EXPECT_TRUE(!n.is_root() || (n.is_doc() && !n.has_key())) _MORE_INFO;
     }
     // vals cannot be containers
     if( ! n.empty() && ! n.is_doc())
     {
-        EXPECT_NE(n.has_val(), n.is_container());
+        EXPECT_NE(n.has_val(), n.is_container()) _MORE_INFO;
     }
     if(n.has_children())
     {
-        EXPECT_TRUE(n.is_container());
-        EXPECT_FALSE(n.is_val());
+        EXPECT_TRUE(n.is_container()) _MORE_INFO;
+        EXPECT_FALSE(n.is_val()) _MORE_INFO;
     }
     // check parent & sibling reciprocity
-    for(NodeRef const s : n.siblings())
+    for(ConstNodeRef s : n.siblings())
     {
-        EXPECT_TRUE(n.has_sibling(s));
-        EXPECT_TRUE(s.has_sibling(n));
-        EXPECT_EQ(s.parent().get(), n.parent().get());
+        EXPECT_TRUE(n.has_sibling(s)) _MORE_INFO;
+        EXPECT_TRUE(s.has_sibling(n)) _MORE_INFO;
+        EXPECT_EQ(s.parent().get(), n.parent().get()) _MORE_INFO;
     }
-    if(n.parent() != nullptr)
+    if(n.parent().readable())
     {
-        EXPECT_TRUE(n.parent().has_child(n));
-        EXPECT_EQ(n.parent().num_children(), n.num_siblings());
+        EXPECT_EQ(n.parent().num_children() > 1, n.has_other_siblings()) _MORE_INFO;
+        EXPECT_TRUE(n.parent().has_child(n)) _MORE_INFO;
+        EXPECT_EQ(n.parent().num_children(), 1 + n.num_other_siblings()) _MORE_INFO;
         // doc parent must be a seq and a stream
         if(n.is_doc())
         {
-            EXPECT_TRUE(n.parent().is_seq());
-            EXPECT_TRUE(n.parent().is_stream());
+            EXPECT_TRUE(n.parent().is_seq()) _MORE_INFO;
+            EXPECT_TRUE(n.parent().is_stream()) _MORE_INFO;
         }
     }
     else
     {
-        EXPECT_TRUE(n.is_root());
+        EXPECT_TRUE(n.is_root()) _MORE_INFO;
     }
     if(n.is_seq())
     {
-        EXPECT_TRUE(n.is_container());
-        EXPECT_FALSE(n.is_map());
-        for(NodeRef const ch : n.children())
+        EXPECT_TRUE(n.is_container()) _MORE_INFO;
+        EXPECT_FALSE(n.is_map()) _MORE_INFO;
+        for(ConstNodeRef ch : n.children())
         {
-            EXPECT_FALSE(ch.is_keyval());
-            EXPECT_FALSE(ch.has_key());
+            EXPECT_FALSE(ch.is_keyval()) _MORE_INFO;
+            EXPECT_FALSE(ch.has_key()) _MORE_INFO;
         }
     }
     if(n.is_map())
     {
-        EXPECT_TRUE(n.is_container());
-        EXPECT_FALSE(n.is_seq());
-        for(NodeRef const ch : n.children())
+        EXPECT_TRUE(n.is_container()) _MORE_INFO;
+        EXPECT_FALSE(n.is_seq()) _MORE_INFO;
+        for(ConstNodeRef ch : n.children())
         {
-            EXPECT_TRUE(ch.has_key());
+            EXPECT_TRUE(ch.has_key()) _MORE_INFO;
         }
     }
     if(n.has_key_anchor())
     {
-        EXPECT_FALSE(n.key_anchor().empty());
-        EXPECT_FALSE(n.is_key_ref());
+        EXPECT_FALSE(n.key_anchor().empty()) _MORE_INFO;
+        EXPECT_FALSE(n.is_key_ref()) _MORE_INFO;
     }
     if(n.has_val_anchor())
     {
-        EXPECT_FALSE(n.val_anchor().empty());
-        EXPECT_FALSE(n.is_val_ref());
+        EXPECT_FALSE(n.val_anchor().empty()) _MORE_INFO;
+        EXPECT_FALSE(n.is_val_ref()) _MORE_INFO;
     }
     if(n.is_key_ref())
     {
-        EXPECT_FALSE(n.key_ref().empty());
-        EXPECT_FALSE(n.has_key_anchor());
+        EXPECT_FALSE(n.key_ref().empty()) _MORE_INFO;
+        EXPECT_FALSE(n.has_key_anchor()) _MORE_INFO;
     }
     if(n.is_val_ref())
     {
-        EXPECT_FALSE(n.val_ref().empty());
-        EXPECT_FALSE(n.has_val_anchor());
+        EXPECT_FALSE(n.val_ref().empty()) _MORE_INFO;
+        EXPECT_FALSE(n.has_val_anchor()) _MORE_INFO;
     }
     // ... add more tests here
 
     // now recurse into the children
-    for(NodeRef const ch : n.children())
+    for(ConstNodeRef ch : n.children())
     {
         test_invariants(ch);
     }
+
+    #undef _MORE_INFO
 }
 
-size_t test_tree_invariants(NodeRef const& n)
+size_t test_tree_invariants(ConstNodeRef const& n)
 {
     auto parent = n.parent();
 
     if(n.get()->m_prev_sibling == NONE)
     {
-        if(parent != nullptr)
+        if(parent.readable())
         {
             EXPECT_EQ(parent.first_child().get(), n.get());
             EXPECT_EQ(parent.first_child().id(), n.id());
@@ -741,14 +785,14 @@ size_t test_tree_invariants(NodeRef const& n)
 
     if(n.get()->m_next_sibling == NONE)
     {
-        if(parent != nullptr)
+        if(parent.readable())
         {
             EXPECT_EQ(parent.last_child().get(), n.get());
             EXPECT_EQ(parent.last_child().id(), n.id());
         }
     }
 
-    if(parent == nullptr)
+    if(!parent.readable())
     {
         EXPECT_TRUE(n.is_root());
         EXPECT_EQ(n.prev_sibling().get(), nullptr);
@@ -756,7 +800,7 @@ size_t test_tree_invariants(NodeRef const& n)
     }
 
     size_t count = 1, num = 0;
-    for(NodeRef const ch : n.children())
+    for(ConstNodeRef ch : n.children())
     {
         EXPECT_NE(ch.id(), n.id());
         count += test_tree_invariants(ch);
@@ -771,15 +815,26 @@ size_t test_tree_invariants(NodeRef const& n)
 void test_invariants(Tree const& t)
 {
 
-    EXPECT_LE(t.size(), t.capacity());
+    ASSERT_LE(t.size(), t.capacity());
     EXPECT_EQ(t.size() + t.slack(), t.capacity());
 
-    if(t.empty()) return;
+    ASSERT_LE(t.arena_size(), t.arena_capacity());
+    ASSERT_LE(t.arena_slack(), t.arena_capacity());
+    EXPECT_EQ(t.arena_size() + t.arena_slack(), t.arena_capacity());
+
+    if(t.empty())
+        return;
 
     size_t count = test_tree_invariants(t.rootref());
     EXPECT_EQ(count, t.size());
 
     check_invariants(t);
+    test_invariants(t.rootref());
+
+    if(!testing::UnitTest::GetInstance()->current_test_info()->result()->Passed())
+    {
+        print_tree(t);
+    }
 
     return;
 #if 0 == 1
@@ -849,12 +904,16 @@ CaseData* get_data(csubstr name)
             replace_all("\r", "", c->src, &tmp);
             cd->unix_style.src_buf.assign(tmp.begin(), tmp.end());
             cd->unix_style.src = to_substr(cd->unix_style.src_buf);
+            cd->unix_style_json.src_buf.assign(tmp.begin(), tmp.end());
+            cd->unix_style_json.src = to_substr(cd->unix_style.src_buf);
         }
         {
             std::string tmp;
             replace_all("\n", "\r\n", cd->unix_style.src, &tmp);
             cd->windows_style.src_buf.assign(tmp.begin(), tmp.end());
             cd->windows_style.src = to_substr(cd->windows_style.src_buf);
+            cd->windows_style_json.src_buf.assign(tmp.begin(), tmp.end());
+            cd->windows_style_json.src = to_substr(cd->windows_style.src_buf);
         }
     }
     else
