@@ -14,12 +14,22 @@
 #include <gtest/gtest.h>
 #include <functional>
 
-#ifdef __GNUC__
-#   pragma GCC diagnostic push
-#   pragma GCC diagnostic ignored "-Wtype-limits"
+
+// no pragma push for these warnings! they will be suppressed in the
+// files including this header (most test files)
+#ifdef __clang__
+#   pragma clang diagnostic ignored "-Wold-style-cast"
+#elif defined(__GNUC__)
+#   pragma GCC diagnostic ignored "-Wold-style-cast"
 #endif
 
-#if defined(_MSC_VER)
+
+#ifdef __clang__
+#   pragma clang diagnostic push
+#elif defined(__GNUC__)
+#   pragma GCC diagnostic push
+#   pragma GCC diagnostic ignored "-Wtype-limits"
+#elif defined(_MSC_VER)
 #   pragma warning(push)
 #   pragma warning(disable: 4296/*expression is always 'boolean_value'*/)
 #   pragma warning(disable: 4389/*'==': signed/unsigned mismatch*/)
@@ -39,6 +49,15 @@ inline void PrintTo(csubstr s, ::std::ostream* os) { os->write(s.str, (std::stre
 
 namespace yml {
 
+inline void PrintTo(NodeType ty, ::std::ostream* os)
+{
+    *os << ty.type_str();
+}
+inline void PrintTo(NodeType_e ty, ::std::ostream* os)
+{
+    *os << NodeType::type_str(ty);
+}
+
 inline void PrintTo(Callbacks const& cb, ::std::ostream* os)
 {
 #ifdef __GNUC__
@@ -69,40 +88,49 @@ void test_compare(Tree const& actual, size_t node_actual,
 void test_arena_not_shared(Tree const& a, Tree const& b);
 
 void test_invariants(Tree const& t);
-void test_invariants(NodeRef const n);
+void test_invariants(ConstNodeRef const& n);
 
 void print_node(CaseNode const& t, int level=0);
 void print_tree(CaseNode const& p, int level=0);
-void print_path(NodeRef const& p);
+void print_path(ConstNodeRef const& p);
 
 
 
 template<class CheckFn>
-void test_check_emit_check(csubstr yaml, CheckFn check_fn)
+void test_check_emit_check(Tree const& t, CheckFn check_fn)
 {
-    Tree t = parse_in_arena(yaml);
     #ifdef RYML_DBG
     print_tree(t);
     #endif
     {
         SCOPED_TRACE("original yaml");
+        test_invariants(t);
         check_fn(t);
     }
-    auto emit_and_parse = [&](const char* identifier){
+    auto emit_and_parse = [&check_fn](Tree const& tp, const char* identifier){
         SCOPED_TRACE(identifier);
-        std::string emitted = emitrs<std::string>(t);
+        std::string emitted = emitrs_yaml<std::string>(tp);
         #ifdef RYML_DBG
         printf("~~~%s~~~\n%.*s", identifier, (int)emitted.size(), emitted.data());
         #endif
-        t = parse_in_arena(to_csubstr(emitted));
+        Tree cp = parse_in_arena(to_csubstr(emitted));
         #ifdef RYML_DBG
-        print_tree(t);
+        print_tree(cp);
         #endif
-        check_fn(t);
+        test_invariants(cp);
+        check_fn(cp);
+        return cp;
     };
-    emit_and_parse("emitted 1");
-    emit_and_parse("emitted 2");
-    emit_and_parse("emitted 3");
+    Tree cp = emit_and_parse(t, "emitted 1");
+    cp = emit_and_parse(cp, "emitted 2");
+    cp = emit_and_parse(cp, "emitted 3");
+}
+
+template<class CheckFn>
+void test_check_emit_check(csubstr yaml, CheckFn check_fn)
+{
+    Tree t = parse_in_arena(yaml);
+    test_check_emit_check(t, check_fn);
 }
 
 
@@ -137,10 +165,15 @@ struct ExpectError
     c4::yml::Callbacks m_tree_prev;
     Location expected_location;
 
+    ExpectError(Location loc={}) : ExpectError(nullptr, loc) {}
     ExpectError(Tree *tree, Location loc={});
     ~ExpectError();
 
+    static void do_check(            std::function<void()> fn, Location expected={}) { do_check(nullptr, fn, expected); }
     static void do_check(Tree *tree, std::function<void()> fn, Location expected={});
+    static void check_success(            std::function<void()> fn) { check_success(nullptr, fn); }
+    static void check_success(Tree *tree, std::function<void()> fn);
+    static void check_assertion(            std::function<void()> fn, Location expected={}) { check_assertion(nullptr, fn, expected); }
     static void check_assertion(Tree *tree, std::function<void()> fn, Location expected={});
 };
 
@@ -413,8 +446,8 @@ public:
 
 public:
 
-    void compare(yml::NodeRef const& n, bool ignore_quote=false) const;
-    void compare_child(yml::NodeRef const& n, size_t pos) const;
+    void compare(yml::ConstNodeRef const& n, bool ignore_quote=false) const;
+    void compare_child(yml::ConstNodeRef const& n, size_t pos) const;
 
     size_t reccount() const
     {
@@ -435,6 +468,7 @@ public:
 typedef enum {
     EXPECT_PARSE_ERROR = (1<<0),
     RESOLVE_REFS = (1<<1),
+    JSON_ALSO = (1<<2), // TODO: make it the opposite: opt-out instead of opt-in
 } TestCaseFlags_e;
 
 
@@ -467,12 +501,22 @@ struct CaseDataLineEndings
     Tree parsed_tree;
 
     size_t numbytes_stdout;
-    std::vector<char> emit_buf;
+    size_t numbytes_stdout_json;
+
+    std::string emit_buf;
     csubstr emitted_yml;
-    std::vector<char> parse_buf;
+
+    std::string emitjson_buf;
+    csubstr emitted_json;
+
+    std::string parse_buf;
     substr parsed_yml;
 
+    std::string parse_buf_json;
+    substr parsed_json;
+
     Tree emitted_tree;
+    Tree emitted_tree_json;
 
     Tree recreated;
 };
@@ -481,19 +525,21 @@ struct CaseDataLineEndings
 struct CaseData
 {
     CaseDataLineEndings unix_style;
+    CaseDataLineEndings unix_style_json;
     CaseDataLineEndings windows_style;
+    CaseDataLineEndings windows_style_json;
 };
 
 
 } // namespace yml
 } // namespace c4
 
-#if defined(_MSC_VER)
+#ifdef __clang__
+#   pragma clang diagnostic pop
+#elif defined(__GNUC__)
+#   pragma GCC diagnostic pop
+#elif defined(_MSC_VER)
 #   pragma warning(pop)
 #endif
 
-#ifdef __GNUC__
-#   pragma GCC diagnostic pop
-#endif
-
 #endif /* _TEST_CASE_HPP_ */
