@@ -3,6 +3,58 @@
 namespace c4 {
 namespace yml {
 
+TEST(github, 268)
+{
+    Tree tree = parse_in_arena(R"(
+        list:
+            - &bar bar
+        map:
+            node: *bar
+    )");
+    tree.resolve();
+    auto root = tree.rootref();
+    ASSERT_TRUE(root["map"].is_map());
+    ASSERT_TRUE(root["map"].has_child("node"));
+    ASSERT_EQ(root["map"]["node"], "bar");
+}
+
+TEST(github, 277)
+{
+    Tree tree = parse_in_arena(R"(
+    A: &A
+        V: 3
+        W: 4
+    B:
+        <<: *A
+        V: 5
+        X: 6
+    )");
+    const char *keys[] = {"V", "W", "X"};
+    const char *vals[] = {"5", "4", "6"};
+    tree.resolve();
+    auto root = tree.rootref();
+    ASSERT_TRUE(root["B"].is_map());
+    size_t num_childs = root["B"].num_children();
+    size_t child = 0;
+    ASSERT_EQ(num_childs, 3);
+    for (const auto node : root["B"].children())
+    {
+        EXPECT_EQ(node.key(), csubstr(keys[child], 1));
+        EXPECT_EQ(node.val(), csubstr(vals[child], 1));
+        child++;
+    }
+    // test whether the tree is corrupted
+    test_invariants(tree);
+    child = num_childs;
+    for (size_t n = tree.last_child(root["B"].id()); n != NONE; n = tree.prev_sibling(n))
+    {
+        ASSERT_NE(child, 0);
+        EXPECT_EQ(tree.key(n), csubstr(keys[child - 1], 1));
+        child--;
+    }
+}
+
+
 TEST(github, 78)
 {
     Tree t = parse_in_arena("{foo: 1, bar: [2, 3]}");
@@ -11,6 +63,7 @@ TEST(github, 78)
     EXPECT_EQ(t["bar"][1].val(), "3");
 }
 
+
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
@@ -40,10 +93,11 @@ TEST(github, 60)
     ASSERT_TRUE(root.is_map());
     ASSERT_TRUE(root.has_child("traits"));
     auto rb = root["traits"]["roleBonuses"][0];
-    ASSERT_TRUE(rb.valid());
+    ASSERT_FALSE(rb.invalid());
+    ASSERT_TRUE(rb.readable());
     EXPECT_EQ(rb["bonus"].val(), "5");
     auto txt = rb["bonusText"];
-    ASSERT_TRUE(txt.valid());
+    ASSERT_TRUE(txt.readable());
     ASSERT_TRUE(txt.is_map());
     EXPECT_TRUE(txt.has_child("de"));
     EXPECT_TRUE(txt.has_child("en"));
