@@ -10,6 +10,8 @@
 namespace c4 {
 namespace yml {
 
+C4_SUPPRESS_WARNING_GCC_CLANG_WITH_PUSH("-Wold-style-cast")
+
 // TODO: add this as a method to csubstr
 bool is_same(csubstr lhs, csubstr rhs)
 {
@@ -532,6 +534,15 @@ TEST(parse_in_arena, overloads)
     }
 }
 
+TEST(parse_in_place, version_numbers)
+{
+    char src1_[] = "{a: 1.2.3}";
+    {
+        Tree tree = parse_in_place(src1_);
+        EXPECT_EQ(tree["a"].val(), "1.2.3");
+    }
+}
+
 } // namespace yml
 } // namespace c4
 
@@ -553,5 +564,8 @@ Case const* get_case(csubstr /*name*/)
 {
     return nullptr;
 }
+
+C4_SUPPRESS_WARNING_GCC_CLANG_POP
+
 } // namespace yml
 } // namespace c4
