@@ -15,6 +15,21 @@ TEST(plain_scalar, issue153_map)
     EXPECT_EQ(t["foo"].val(), "A");
 }
 
+TEST(plain_scalar, test_suite_BS4K)
+{
+    Tree t;
+    ExpectError::do_check(&t, [&]{
+        t = parse_in_arena(R"(word1  # comment
+word2
+word3
+)");
+    });
+    ExpectError::do_check(&t, [&]{
+        t = parse_in_arena(R"(word1  # comment
+word2
+)");
+    });
+}
 
 TEST(plain_scalar, test_suite_7TMG)
 {
@@ -29,7 +44,7 @@ word1
 )";
     test_check_emit_check(yaml, [](Tree const &t){
         EXPECT_TRUE(t.rootref().is_stream());
-        NodeRef doc = t.rootref().first_child();
+        ConstNodeRef doc = t.rootref().first_child();
         ASSERT_TRUE(doc.is_doc());
         ASSERT_TRUE(doc.is_val());
         EXPECT_EQ(doc.val(), "word1");
